@@ -177,6 +177,31 @@ Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
 #define SUSE_VERSION 0
 #endif /* SUSE_VERSION */
 
+/* SUSE kernel version comparison macros.
+ * SUSE kernel versioning format may be A.B.C-D.E.F and standard KERNEL_VERSION
+ * macro covers only the first 3 subversions.
+ * Using 20bit per subversion, as in some cases, subversion D may be a large
+ * number (6 digits).
+ */
+#define SUSE_KERNEL_VERSION_16BIT(SV1, SV2, SV3) ((SV1 << 40) | (SV2 << 20) | (SV3))
+#define SUSE_KERNEL_VERSION_MAJOR(SV1, SV2, SV3) SUSE_KERNEL_VERSION_16BIT(SV1, SV2, SV3)
+#define SUSE_KERNEL_VERSION_MINOR(SV1, SV2, SV3) SUSE_KERNEL_VERSION_16BIT(SV1, SV2, SV3)
+
+#define SUSE_KERNEL_VERSION_GTE(SV1, SV2, SV3, SV4, SV5, SV6) \
+	((SUSE_KERNEL_VERSION_MAJOR(SUSE_KERNEL_SUBVERSION_1, SUSE_KERNEL_SUBVERSION_2, SUSE_KERNEL_SUBVERSION_3) > \
+	  SUSE_KERNEL_VERSION_MAJOR((SV1), (SV2), (SV3))) || \
+	 (SUSE_KERNEL_VERSION_MAJOR(SUSE_KERNEL_SUBVERSION_1, SUSE_KERNEL_SUBVERSION_2, SUSE_KERNEL_SUBVERSION_3) == \
+	  SUSE_KERNEL_VERSION_MAJOR((SV1), (SV2), (SV3)) && \
+	  SUSE_KERNEL_VERSION_MINOR(SUSE_KERNEL_SUBVERSION_4, SUSE_KERNEL_SUBVERSION_5, SUSE_KERNEL_SUBVERSION_6) >= \
+	  SUSE_KERNEL_VERSION_MINOR((SV4), (SV5), (SV6))))
+
+#define SUSE_KERNEL_VERSION_LTE(SV1, SV2, SV3, SV4, SV5, SV6) \
+	((SUSE_KERNEL_VERSION_MAJOR(SUSE_KERNEL_SUBVERSION_1, SUSE_KERNEL_SUBVERSION_2, SUSE_KERNEL_SUBVERSION_3) < \
+	  SUSE_KERNEL_VERSION_MAJOR((SV1), (SV2), (SV3))) || \
+	 (SUSE_KERNEL_VERSION_MAJOR(SUSE_KERNEL_SUBVERSION_1, SUSE_KERNEL_SUBVERSION_2, SUSE_KERNEL_SUBVERSION_3) == \
+	  SUSE_KERNEL_VERSION_MAJOR((SV1), (SV2), (SV3)) && \
+	  SUSE_KERNEL_VERSION_MINOR(SUSE_KERNEL_SUBVERSION_4, SUSE_KERNEL_SUBVERSION_5, SUSE_KERNEL_SUBVERSION_6) <= \
+	  SUSE_KERNEL_VERSION_MINOR((SV4), (SV5), (SV6))))
 
 /******************************************************************************/
 /**************************** RHEL macros *************************************/
@@ -881,32 +906,16 @@ static inline int numa_mem_id(void)
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
 #define ENA_LINEAR_FRAG_SUPPORTED
-static __always_inline struct sk_buff*
-ena_build_skb(void *data, unsigned int frag_size)
-{
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 12, 0)
-	return napi_build_skb(data, frag_size);
-#else
-	return build_skb(data, frag_size);
-#endif
-}
-#endif
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 6, 0) && \
-	!(defined(RHEL_RELEASE_CODE) && RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(7, 3)) && \
-	!(defined(UBUNTU_VERSION_CODE) && UBUNTU_VERSION_CODE >= UBUNTU_VERSION(4, 2, 0, 42))
-static __always_inline
-void napi_consume_skb(struct sk_buff *skb, int budget)
-{
-	dev_kfree_skb_any(skb);
-}
 #endif
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 17, 0)
 #define ENA_NETDEV_LOGS_WITHOUT_RV
 #endif
 
-#if defined(ENA_XDP_SUPPORT) && LINUX_VERSION_CODE < KERNEL_VERSION(5, 12, 0)
+#if defined(ENA_XDP_SUPPORT) && \
+	(LINUX_VERSION_CODE < KERNEL_VERSION(5, 12, 0) && \
+	!(defined(SUSE_VERSION) && (SUSE_VERSION == 15 && SUSE_PATCHLEVEL == 3) && \
+	 SUSE_KERNEL_VERSION_GTE(5, 3, 18, 150300, 59, 49)))
 static __always_inline void
 xdp_init_buff(struct xdp_buff *xdp, u32 frame_sz, struct xdp_rxq_info *rxq)
 {
@@ -928,7 +937,7 @@ xdp_prepare_buff(struct xdp_buff *xdp, unsigned char *hard_start,
 	xdp->data_meta = meta_valid ? data : data + 1;
 }
 
-#endif /* defined(ENA_XDP_SUPPORT) && LINUX_VERSION_CODE <= KERNEL_VERSION(5, 12, 0) */
+#endif /* defined(ENA_XDP_SUPPORT) && (LINUX_VERSION_CODE <= KERNEL_VERSION(5, 12, 0) && !SUSE_VERSION(...)) */
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(5, 13, 0)
 #define ethtool_sprintf(data, fmt, args...)			\
@@ -944,7 +953,9 @@ xdp_prepare_buff(struct xdp_buff *xdp, unsigned char *hard_start,
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(5, 15, 0) && \
 	!(defined(RHEL_RELEASE_CODE) && RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8, 6)) && \
-	!(defined(SUSE_VERSION) && (SUSE_VERSION == 15 && SUSE_PATCHLEVEL >= 4))
+	!(defined(SUSE_VERSION) && (SUSE_VERSION == 15 && SUSE_PATCHLEVEL >= 4)) && \
+	!(defined(SUSE_VERSION) && (SUSE_VERSION == 15 && SUSE_PATCHLEVEL == 3) && \
+	  SUSE_KERNEL_VERSION_GTE(5, 3, 18, 150300, 59, 43))
 static inline void eth_hw_addr_set(struct net_device *dev, const u8 *addr)
 {
 	memcpy(dev->dev_addr, addr, ETH_ALEN);
